// avr-compat.h : 
// defines for compatibility to AVR/GCC code (this is -C-, but with GCC Extensions...)

#pragma once

// common functions, also avail on gcc
#include <stdlib.h>
#include <string.h>

// GCC has standard int types, often used for AVR programming, so we give them here
#include "stdint.h"

// AVR programming uses 16-bit ints, so we are
typedef unsigned short uint;
typedef short int16;
typedef unsigned char byte;

// AVR programming progmen emulation
typedef void prog_void;
typedef char prog_char;
typedef unsigned char prog_uchar;
typedef int8_t    prog_int8_t   ;
typedef uint8_t   prog_uint8_t  ;
typedef int16_t   prog_int16_t  ;
typedef uint16_t  prog_uint16_t ;
typedef int32_t   prog_int32_t  ;
typedef uint32_t  prog_uint32_t ;
typedef int64_t   prog_int64_t  ;
typedef uint64_t  prog_uint64_t ;

#define pgm_read_byte_near(address_short) *(address_short)
#define pgm_read_word_near(address_short) *(address_short)
#define pgm_read_dword_near(address_short) *(address_short)
#define pgm_read_float_near(address_short) *(address_short)

#define pgm_read_byte(address_short) (*(address_short))
#define pgm_read_word(address_short) (*(address_short))
#define pgm_read_dword(address_short) (*(address_short))
#define pgm_read_float(address_short) (*(address_short))

#define memchr_P        memchr
#define memcmp_P        memcmp 
#define memccpy_P				memccpy
#define memcpy_P				memcpy
#define memmem_P				memmem
#define memrchr_P				memrchr
#define strcat_P				strcat
#define strchr_P				strchr
#define strchrnul_P			strchrnul
#define strcmp_P				strcmp
#define strcpy_P				strcpy
#define strcasecmp_P		strcasecmp
#define strcasestr_P		strcasestr
#define strcspn_P				strcspn
#define strlcat_P				strlcat
#define strlcpy_P				strlcpy
#define strlen_P				strlen
#define strnlen_P				strnlen
#define strncmp_P				strncmp
#define strncasecmp_P		strncasecmp
#define strncat_P				strncat
#define strncpy_P				strncpy
#define strpbrk_P				strpbrk
#define strrchr_P				strrchr
#define strsep_P				strsep
#define strspn_P				strspn
#define strstr_P				strstr
#define strtok_P				strtok
#define strtok_rP				strtok_r

// AVR programming GCC binary constanst emulation!
#define HEX__(n) 0x##n##LU
#define B8__(x) ((x&0x0000000FLU)?1:0)\
               +((x&0x000000F0LU)?2:0)\
              +((x&0x00000F00LU)?4:0)\
               +((x&0x0000F000LU)?8:0)\
               +((x&0x000F0000LU)?16:0)\
               +((x&0x00F00000LU)?32:0)\
               +((x&0x0F000000LU)?64:0)\
               +((x&0xF0000000LU)?128:0)
#define B8(d) ((unsigned char)B8__(HEX__(d)))
#define B16(dmsb,dlsb) (((unsigned short)B8(dmsb)<<8) + B8(dlsb))
#define B32(dmsb,db2,db3,dlsb) (((unsigned long)B8(dmsb)<<24) + ((unsigned long)B8(db2)<<16) + ((unsigned long)B8(db3)<<8) + B8(dlsb))

//B8(011100111)
//B16(10011011,10011011)
//B32(10011011,10011011,10011011,10011011)

#pragma warning(disable : 4996)  // posix name deprecated in VC++



