/*
 * avrpong.c
 *
 * "free" Pong for Ping-Pong Board from Conrad
 *
 * Ready for AVR Studio 4 or WinAVR
 * Modified multiplaxing display procedure based on the code from
 * Laufschrift program written by Sascha Bader
 *
 * Ver.     Date         Author           Comments
 * -------  ----------   --------------   ------------------------------
 * 1.00	    25.12.2009   Piotr Platek     initial 
 * 1.00-PC  28.02.2010   Hans Eckardt     adapted to run in the PC programming framework :-) "PongSimu"
 * 1.01-PC  04.03.2010   Hans Eckardt     it works now in the emulator, not the real thing though
 *
 *
 * Feel free to provide me any feedback 
 ***************************************************************************
 **
 **  This program is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU General Public License
 **  as published by the Free Software Foundation; either version 2
 **  of the License, or (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software Foundation, 
 **  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ***************************************************************************
 */

//#define F_CPU 8000000UL	                       /* CPU frequency for delay macro) */


/* ---------------------------------------------------------------------------- 
 * Macros for X-Y simple graphic functions
 */

#define GetPixel(x,y) leds[x]&(1<<(9-y))     /* Macro: "Pixel" status read */
#define SetPixel_(x,y) leds[x]|=1<<(9-y)      /* Macro: "Pixel" set */
#define ClearPixel_(x,y) leds[x]&=~(1<<(9-y)) /* Macro: "Pixel" clear */

/* ---------------------------------------------------------------------------- 
 * Includes																	 
 */

/*
#include <inttypes.h>		                   	
#include <avr/interrupt.h>                     	
#include <avr/io.h>
#include <util/delay.h>		                   	
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/pgmspace.h>                      	
*/

#include "avr-stuff.h"                           	// simulated hardware lives here

#include "display.h"                           	
#include "fonts.h"
#include "texts.h"

#ifdef _DEBUG // emulation debugging
#include <stdio.h>
extern "C" __declspec(dllimport) void __stdcall OutputDebugStringA(const char* lpOutputString);
#pragma comment(lib,"kernel32")
#endif

// sync the pixbuf here with the emulation...

inline void SetPixel(uint x, uint y)
{
	SetPixel_(x,y);
	avrSetPixel(x, y, 2);
}

inline void ClearPixel(uint x, uint y)
{
	ClearPixel_(x,y);
	avrSetPixel(x, y,0);
}


inline void updateLedColumn(int col)
{
	for (byte y= 0; y < SHIGHT; ++y)
		avrSetPixel(col, SHIGHT-1-y, (leds[col] & (1<<y)) ? 2 : 0);
}



/* ---------------------------------------------------------------------------- 
 * Global variables
 */
 
uint16_t BallSpeed = BALLSPD;  			// Initial speed of the ball 
volatile uint16_t BallDelay;			// Delay counter for ball speed calculation 
BallType BallPos;						// Position of the Ball
BallType BallDxy;						// Motion of the ball in terms of dx & dy
BallType BallTmp;						// Position of the Ball for calculation


uint16_t LPoti=0,  RPoti=0;				// Current ADC values for left and right Poti
uint16_t LPotiP=0, RPotiP=0;			// Previous ADC values for left and right Poti
int16_t  LPotiD=0, RPotiD=0, PotiD=0;	// Poti dynamic = Previous - Current in time frame PotiT
volatile uint16_t PotiDelay;	// Time delta for pad movement dynamic calculation
int16_t  dx;							// For pad's corner calculation
uint8_t   LCounter, RCounter;			// Game Couners for points collecting


/* ---------------------------------------------------------------------------- 
 * Function prototypes 														 
 */

void LedScroll(uint8_t *c, int8_t lx, int8_t rx, int8_t y, uint16_t t);
void pgm_LedScroll(prog_uint8_t *Cpstr);
void LedPrint(uint8_t *c, int8_t x, int8_t y);
void UpdateBallPos(void);
void GameStart(void);
void ReadBallSpeed(uint16_t);
void LedCLS(void);
void StandBy(void);
void UpdatePoti(void);
// void Initialize();



/* ---------------------------------------------------------------------------- 
 * Main Funktion 															 
 */

int main(void)
{
	LedCLS();							// Clear screen									
	pgm_LedScroll(str_00);				// Scroll the first Welcome text

	GameStart();						// Setup game parameters
	BallDelay = ticks();
	PotiDelay= BallDelay;

	// Main Loop
	while (!avrEndRun())							// Typical loop for Controler
	{
		// Update both LPoti & RPoti Values
		UpdatePoti();

		// Draw Pads
		leds[LPadCol] = ( (PadDef << (LPoti / PotiScale) ) & PadMask) >> 2 ;
		updateLedColumn(LPadCol);

		leds[RPadCol] = ( (PadDef << (RPoti / PotiScale) ) & PadMask) >> 2 ;
		updateLedColumn(RPadCol);

		if ((BallPos.x == SWIDTH-1) || (BallPos.x == 0)) // Update Pad's column if ball's missed pad
		{
			SetPixel(BallPos.x, BallPos.y);
		}
		avrTriggerRepaint();

		// Draw Ball
		if (uint(ticks() - BallDelay) > BallSpeed) // Check if the delay for next ball point has elapsed
		{
			// Calculate ball's next position
			UpdateBallPos();

			// Check if outside of the screen
			if ( (BallPos.x >= SWIDTH-1) || (BallPos.x <= 0 ) )	
			{   // Ball is outside the screen
				if (BallPos.x >= SWIDTH-1)	// Ball crossed the right border
				{	
					LCounter++;
					BallTmp.x = SWIDTH-2;
					BallDxy.x = -1;
				}
				else 						// Ball crossed the right border
				{  
					RCounter++;
					BallTmp.x = 1;
					BallDxy.x = +1;
				}
				// Display current game result
				LedCLS();
				LedPrint((uint8_t *)itoa(LCounter, (char *)TextBuffer, 10), 2, 3);	
				LedPrint((uint8_t *)":", 6, 3);
				LedPrint((uint8_t *)itoa(RCounter, (char *)TextBuffer, 10), 8, 3);
				_delay_ms(2000);
				LedCLS();
				PotiDelay = ticks();
				
				// Check if we reach game over counter
				if ( (LCounter>=GameOverCounter) || (RCounter>=GameOverCounter) )
				{
					// Game Over - display text and go to Standby mode
					pgm_LedScroll(str_03);		
					LedCLS();
					_delay_ms(2000);
					StandBy();
					
					// Start the game after coming back from Standby mode
					LedCLS();
					pgm_LedScroll(str_04);
					GameStart();
					UpdateBallPos();
				} 
				else			// not yet game over start next ball
				{
				BallTmp.y = 5;
				BallDxy.y = 0;
				}
			}
			else // Colision with pad or walls
			{	
				// calculate movement for the upper and lower walls colision
				if ( ( (BallTmp.y > SHIGHT-1) || (BallTmp.y < 0) ) && (BallDxy.y) ) 
				{

					BallDxy.y *= -1; // simple mirror in Y axis
					UpdateBallPos();
				}
				
				// check and calculate a case of a pad's colision
				if ((BallTmp.x >= SWIDTH-1) || (BallTmp.x <= 0)) 
				{
					// Check the side of possible colision and setup the calculation support data
					if   (BallTmp.x >= SWIDTH-1)	// Colision with right pad
					{
						dx = 1;						// to check the angle o the colision with right pad
						PotiD  = RPotiD;			// we will calculate Poti dynamic for the right pad
					}
					else							// Colision with left pad
					{
						dx = -1;					// to check the angle o the colision with left pad
						PotiD  = LPotiD;			// we will calculate Poti dynamic for the left pad
					}

					// Check if ball hits pad
					if (GetPixel( (BallPos.x+dx), BallPos.y)) 	// colision with the middle of a pad
					{
						BallDxy.x *= -1;						// mirror movement on the x axis
						if ( 
							 ( ( abs(PotiD) > Lev1) && !BallDxy.y)
							 || (abs(PotiD) > Lev2)
						   ) // if vertical move BallDxy==0 and dynamic > Lev1 or dynamic > Lev2
						{
							//int old= BallDxy.y;
							BallDxy.y -= (PotiD>0?-1:1);   // turn the ball in direction of pad's move (HE: not strong, only parial)
							//char buf[80];
							//sprintf(buf, "dy old %d, new : %d\r\n", old, BallDxy.y);
							//OutputDebugStringA(buf); 

							//HE: clamp dy, as partial change now
							if (BallDxy.y > 1) BallDxy.y=1;
							else if (BallDxy.y < -1) BallDxy.y= -1;
						}
						UpdateBallPos();
					}
					else if (GetPixel(BallTmp.x, BallTmp.y))  // has to be a corner, 180 bounce
					{
						BallDxy.x *= -1;  
						BallDxy.y *= -1;


						if ( abs(PotiD) > Lev2 ) 
						// For corenr and dynamic > Lev2 generate a slip and turn the vertical ball 
						{
							BallDxy.y = 0;
						}

						UpdateBallPos();
					}
				}
			}
			ClearPixel(BallPos.x, BallPos.y);		// clear pixel at the old ball position
			BallPos = BallTmp;						// new calculated position will be the new position
			SetPixel(BallPos.x, BallPos.y);			// draw ball at new position
			// Restore ball's delay
			BallDelay = ticks(); 		
		}
		else
			_delay_ms(5);
	}
}


void UpdatePoti(void)
{
/* Update Results from ADAC for Pads stearing only when ADAC is ready ADSC 1->0
 * to save the time no function parameters
 * side efect update of global variables: 
 * LPoti, RPoti, LPotiP, LPotiD, RPotiP, RPotiD
*/
	RPoti = avrGetRightADC();
	LPoti = avrGetLeftADC();

	if ( uint(ticks() - PotiDelay) > POTID) // check if the time delta for Poti calculation elapsed
	{
		PotiDelay = ticks();			// update dynamic of the Poti movement
		LPotiD = LPotiP - LPoti;
		RPotiD = RPotiP - RPoti;
		LPotiP = LPoti;
		RPotiP = RPoti;
		//char buf[80];
		//sprintf(buf, "potil : %d, potir : %d\r\n", LPotiD, RPotiD);
		//OutputDebugStringA(buf); 
	}
	//else
	//{
	//	if (abs(LPotiP - LPoti) > abs(LPotiD))
	//		LPotiD = LPotiP - LPoti;
	//	if (abs(RPotiP - RPoti) > abs(RPotiD))
	//		RPotiD = RPotiP - RPoti;
	//}
}


void UpdateBallPos(void)
{
/* Update interim position of the ball 
 * to save the time no parameters
 * side efect update of global variable: 
 * BallTmp
*/

  BallTmp.x = BallPos.x + BallDxy.x;
  BallTmp.y = BallPos.y + BallDxy.y;
}


void GameStart(void)
{
/* Prepare the game parameters after power on and standby mode 
 * side efect update of global variable: 
 * LPoti, BallSpeed, BallPos, BallDxy
 * fix me: no need to work on global variables
*/

	int8_t i;

	LPoti = 0;
	LedPrint((uint8_t *)":", 0, 3);
	// Set up a ball speed
	while( LPoti < 700)  
	{
		if (avrEndRun()) return;
		UpdatePoti();
		BallSpeed = (RPoti / 100 + 1)*10;
		LedPrint((uint8_t *)itoa(BallSpeed, (char *)TextBuffer, 10),2, 3);
		BallSpeed /= 4;
		_delay_ms(150);
		LedPrint((uint8_t *)"   ", 2, 3);
	}
	BallSpeed *= 8; 			// !! fix this in relation to the Timer TCCR0 initial it was 64, now it was decrease to 8
	BallDelay = BallSpeed;

	LedPrint((uint8_t *)"< >",0,3);
	// check who will start. Better Poti dynamic wins
	_delay_ms(300);
	PotiDelay= ticks();

	uint16_t lpd=0;  // accumulate dynamics for simulation
	uint16_t rpd=0;
	for (i=0; i < 100; i++) 
	{
	   UpdatePoti();
	   _delay_ms(15);
		 if (abs(LPotiD) > lpd)
			 lpd= abs(LPotiD);
		 if (abs(RPotiD) > rpd)
			 rpd= abs(RPotiD);
	}
	if (lpd > rpd)
	{ 	
		BallPos.x = 1;
		BallDxy.x = +1;
		LedPrint((uint8_t *)">>>",0,3);			
	}
	else
	{
		BallPos.x = SWIDTH-2;
		BallDxy.x = -1;		
		LedPrint((uint8_t *)"<<<",0,3);			
	}
	BallPos.y = 5;
	BallDxy.y = 0;
	BallTmp.x = 0;
	BallTmp.y = 0;
	LCounter = 0;
	RCounter = 0;
	_delay_ms(1500);
	LedCLS();
	LedPrint((uint8_t *)strcpy_P((char*)TextBuffer, (char*)str_02), 2, 3);
	_delay_ms(1500);
	LedCLS();
}

void LedCLS(void)
{
	/* "Clear Screen" */
	for (int8_t i = 0; i < SWIDTH; leds[i++] = 0);

	for (int x= 0; x < SWIDTH; ++x)
		updateLedColumn(x);

	avrTriggerRepaint();
}

void LedPrint(uint8_t *c, int8_t x, int8_t y)
{
/* Print string on the screen, no scroll, text outside of the screen will not be printed
   c: pointer to the regular string to be printed
   x,y position of the first character from the screen (left lower corner of the first character)
*/
  uint8_t i, fn, FontWidth;
  uint16_t 	ResetColV;
  uint8_t *text;

  ResetColV = ResetCol << (FONTHEIGHT-y);  // Calculate a mask for clearing place for the new charactr's column
  for (text=c; (*text); text++)			   // for all characters in in the string to be printed
  {
	fn = *text - FirstASCIICharacter;	   // get the index of the character in the ASCII table from fonts.h
	FontWidth = pgm_read_byte(&font[fn][FONTWIDTHIDX])+FONTDEFIDX;  // read the fonth width based on the font 'sindex
	for (i=FONTDEFIDX ; i < FontWidth ; i++)						// Draw font's columns
	{
        if ((x >= 0) && (x < SWIDTH))								// Draw only on the screen, fix me: do not process the text any longer
		{
		  leds[x] &= ~ResetColV;									// clear the place for the font's column
		  leds[x] |= pgm_read_byte(&font[fn][i]) << (FONTHEIGHT-y); // draw one font's column data
			updateLedColumn(x);
		}
		x++;														// next column on the screen
	}
	x++;  															// after one character make one column empty  
  }
	avrTriggerRepaint();
}

void pgm_LedScroll(prog_uint8_t *Cpstr)
{
/* Scroll a text from the Progam memory using LedScroll function
 * Cpstr - pointer to the string at the Porogram memory 
*/
  int8_t i;
  uint8_t ch;
  
	if (Cpstr)	// as long as not '\0'
	{
		if (avrEndRun()) return;
		i = 0;
		while ( (ch=pgm_read_byte(Cpstr++)) ) 			// read character from program memory
		{
			if ( i == MaxSRAMTextBuffer ) 				// check if we reach the end of local SRAM buffer
			{
				TextBuffer[i] = '\0';					// add '\0' to the local buffer
				i = 0;
				LedScroll(TextBuffer, 0, 11, 3, 100);	// scroll the buffer on the screen
				TextBuffer[i++] = ch;
			}
			else
			{
				TextBuffer[i++] = ch;					// store the character from program mem. in local SRAm buffer
			}
		}
		if ( i == MaxSRAMTextBuffer ) 					// all characters read from program mem read exectly in the sram buffer
		{
			TextBuffer[i] = '\0';						// finish the local buffer with '\0' character
		}
		else											
		{
			TextBuffer[i++] = ch;						// add '\0' character to print the rest of the local buffer
			TextBuffer[i] = '\0';
		}
		LedScroll(TextBuffer, 0, 11, 3, 100);			// scroll the rest after processing main loop
	}
}

void LedScroll(uint8_t *c, int8_t lx, int8_t rx, int8_t y, uint16_t t)
{
/* Scroll a text from the SRAM memory 
 * c: pointer to the string at the SRAM memory 
 * lx, rx: left and right coordination of the scroll window
 * y: y position of the scroll window
 * t: scroll speed 
*/

  uint8_t fcol, scol, fn, FontWidth;
  uint16_t ResetColV;
  uint8_t *text;

  
  if ( (rx >= 0) && (rx < SWIDTH) && (lx >= 0) && (lx < SWIDTH) && (lx < rx) )  // scroll only in display and for xl < xr
  {
	ResetColV = ResetCol << (FONTHEIGHT-y);  // see LedPrint
	for (text=c; (*text); text++)
	{
		fn = *text - FirstASCIICharacter;
		FontWidth = pgm_read_byte(&font[fn][FONTWIDTHIDX])+FONTDEFIDX;
		for (fcol=FONTDEFIDX; fcol < FontWidth; fcol++) 
		{    
			for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left
			{
				leds[scol] &= ~ResetColV;
				leds[scol] |= leds[scol+1] & ResetColV;
				updateLedColumn(scol);
			}
			leds[rx] &= ~ResetColV;
			leds[rx] |= pgm_read_byte_near(&font[fn][fcol])<<(FONTHEIGHT-y);
			updateLedColumn(rx);
			avrTriggerRepaint();
			_delay_ms(t);	
		}
		for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left after one char
		{
			leds[scol] &= ~ResetColV;
			leds[scol] |= leds[scol+1] & ResetColV;
			updateLedColumn(scol);
		}
		leds[rx] &= ~ResetColV;  // and add one empty column
		updateLedColumn(rx);

		avrTriggerRepaint();
	}
  }
}


void StandBy(void)
{
/* Initiate a standby mode for the Controler */
	for (;;) {
		_delay_ms(500);
		if (avrEndRun()) return;
	}
}


/* ---------------------------------------------------------------------------- 
 * Interrupt Procedures
 */

#if 0
{
	if (BallDelay) BallDelay--;	   	/* counter decrease as long as it's not 0 */
	if (PotiDelay) PotiDelay--;		/* counter decrease as long as it's not 0 */
}
#endif